@@ -34,6 +34,8 @@
 
 #include "GNOME_Media_CDDBSlave2.h"
 
+extern void destroy_cache_hashTable (void);
+
 static GNOME_Media_CDDBTrackEditor track_editor = CORBA_OBJECT_NIL;
 
 static gboolean position_auto_update=TRUE;
@@ -504,10 +506,9 @@ status_ok (GnomeCD *gcd,
 
 	case GNOME_CDROM_AUDIO_PLAY:
 		/* Change the play button to pause */
-		aob = gtk_widget_get_accessible (GTK_WIDGET (gcd->play_b));
-		atk_object_set_name (aob, _("Pause"));
-
 		if (gcd->current_image != gcd->pause_image) {
+			aob = gtk_widget_get_accessible (GTK_WIDGET (gcd->play_b));
+			atk_object_set_name (aob, _("Pause"));
 			gtk_container_remove (GTK_CONTAINER (gcd->play_b),
 					      gcd->play_image);
 			gtk_container_add (GTK_CONTAINER (gcd->play_b),
@@ -567,9 +568,9 @@ status_ok (GnomeCD *gcd,
 
 	case GNOME_CDROM_AUDIO_PAUSE:
 		/* Change the play button to pause */
-		aob = gtk_widget_get_accessible (GTK_WIDGET (gcd->play_b));
-		atk_object_set_name (aob, _("Play"));
 		if (gcd->current_image != gcd->play_image) {
+			aob = gtk_widget_get_accessible (GTK_WIDGET (gcd->play_b));
+			atk_object_set_name (aob, _("Play"));
 			gtk_container_remove (GTK_CONTAINER (gcd->play_b),
 					      gcd->pause_image);
 			gtk_container_add (GTK_CONTAINER (gcd->play_b),
