@@ -33,6 +33,9 @@ static char *cd_option_device = NULL;
 static gboolean cd_option_unique = FALSE;
 static gboolean cd_option_play = FALSE;
 
+/* if env var GNOME_CD_DEBUG is set,
+ * g_warning the given message, and if there was an error, the error message
+ */
 void
 gcd_warning (const char *message,
 	     GError *error)
@@ -44,6 +47,35 @@ gcd_warning (const char *message,
 	g_warning (message, error ? error->message : "(None)");
 }
 
+/* if env var GNOME_CD_DEBUG is set, g_print the given message */
+void
+gcd_debug (const gchar *format,
+           ...)
+{
+	va_list args;
+	gchar *string;
+
+	if (debug_mode == FALSE) {
+		return;
+	}
+	va_start (args, format);
+	string = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	g_print ("DEBUG: gnome-cd: %s\n", string);
+	g_free (string);
+}
+
+/*
+ * Sets the window title based on the passed artist and track name.
+ * If artist and track is NULL, then "CD Player" is displayed.
+ * If either one is NULL, display the other.
+ * If neither are NULL, display "(artist) - (title)"
+ *
+ * FIXME: it might be nice to change this function so that it takes
+ * artist, track, album, status, so we can decide better what to display.
+ * Right now, sometimes this is called with the disc title instead of
+ * the track title */
 void
 gnome_cd_set_window_title (GnomeCD *gcd,
 			   const char *artist,
@@ -52,11 +84,14 @@ gnome_cd_set_window_title (GnomeCD *gcd,
 	char *title;
 	const char *old_title;
 
-	if (artist == NULL ||
-	    track == NULL) {
+	if (artist == NULL && track == NULL) {
 		title = g_strdup (_("CD Player"));
+	} else if (artist == NULL) {
+		title = g_strdup (track);
+	} else if (track == NULL) {
+		title = g_strdup (artist);
 	} else {
-		title = g_strconcat (track, " - ", artist, NULL);
+		title = g_strconcat (artist, " - ", track, NULL);
 	}
 	/*
 	 * Call gtk_window_set_title only if the title has changed
@@ -227,23 +262,6 @@ make_button_from_widget (GnomeCD *gcd,
 	return button;
 }
 
-static GtkWidget *
-make_button_from_pixbuf (GnomeCD *gcd,
-			 GdkPixbuf *pixbuf,
-			 GCallback func,
-			 const char *tooltip,
-			 const char *shortname)
-{
-	GtkWidget *pixmap;
-
-	g_return_val_if_fail (gcd != NULL, NULL);
-	g_return_val_if_fail (pixbuf != NULL, NULL);
-		
-	pixmap = gtk_image_new_from_pixbuf (pixbuf);
-
-	return make_button_from_widget (gcd, pixmap, func, tooltip, shortname);
-}
-
 static GtkWidget *
 make_button_from_stock (GnomeCD *gcd,
 			const char *stock,
@@ -283,11 +301,14 @@ window_destroy_cb (GtkWidget *window,
 {
 	GnomeCD *gcd = data;
 
+	/* Make sure we stop playing the cdrom first */
+	gnome_cdrom_stop(gcd->cdrom, NULL);
+
 	/* Before killing the cdrom object, do the shutdown on it */
 	if (gcd->preferences->stop_eject) {
 		gnome_cdrom_eject (gcd->cdrom, NULL);
 	}
-		
+
 	/* Unref the cddb slave */
 	cddb_close_client ();
 
@@ -537,6 +558,8 @@ init_player (const char *device_override)
 			  G_CALLBACK (loopmode_changed_cb), gcd);
 	g_signal_connect (G_OBJECT (gcd->display), "playmode-changed",
 			  G_CALLBACK (playmode_changed_cb), gcd);
+	g_signal_connect (G_OBJECT (gcd->display), "style_set",
+			  G_CALLBACK (cd_display_set_style), NULL);
 
 	/* Theme needs to be loaded after the display is created */
 	gcd->theme = (GCDTheme *)theme_load (gcd, gcd->preferences->theme_name);
@@ -724,7 +747,7 @@ save_session(GnomeClient        *client,
     return TRUE;
 }
 
-static gint client_die(GnomeClient *client,
+static void client_die(GnomeClient *client,
 		       GnomeCD *gcd)
 {
 	gtk_widget_destroy (gcd->window);
@@ -780,7 +803,7 @@ register_stock_icons (void)
 int 
 main (int argc, char *argv[])
 {
-	static const struct poptOption cd_popt_options [] = {
+	static struct poptOption cd_popt_options [] = {
 		{ "device", '\0', POPT_ARG_STRING, &cd_option_device, 0,
 		  N_("CD device to use"), NULL },
 		{ "unique", '\0', POPT_ARG_NONE, &cd_option_unique, 0,
