@@ -172,12 +172,12 @@ linux_cdrom_update_cd (GnomeCDRom *cdrom)
 	priv = lcd->priv;
 
 	if (linux_cdrom_open (lcd, &error) == FALSE) {
-		g_warning ("Error opening CD");
+		g_message ("Error opening CD");
 		return;
 	}
 
 	if (ioctl (cdrom->fd, CDROMREADTOCHDR, priv->tochdr) < 0) {
-		g_warning ("Error reading CD header");
+		g_message ("Error reading CD header");
 		linux_cdrom_close (lcd);
 
 		return;
@@ -208,12 +208,17 @@ linux_cdrom_update_cd (GnomeCDRom *cdrom)
 	if (ioctl (cdrom->fd, CDROMREADTOCENTRY, &tocentry) < 0) {
 		g_warning ("Error getting leadout");
 		linux_cdrom_invalidate (lcd);
+		g_free (priv->track_info);
+		priv->track_info = NULL;
 		return;
 	}
 	ASSIGN_MSF (priv->track_info[priv->number_tracks].address, tocentry.cdte_addr.msf);
 	calculate_track_lengths (lcd);
 
 	linux_cdrom_close (lcd);
+	//g_free (priv->track_info);
+	//priv->track_info = NULL;
+
 	return;
 }
 
@@ -437,6 +442,7 @@ linux_cdrom_play (GnomeCDRom *cdrom,
 	}
 
 	g_free (status);
+	
 	/* Get the status again: It might have changed */
 	if (gnome_cdrom_get_status (GNOME_CDROM (lcd), &status, error) == FALSE) {
 		linux_cdrom_close (lcd);
@@ -476,7 +482,9 @@ linux_cdrom_play (GnomeCDRom *cdrom,
 			msf.cdmsf_sec0 = status->absolute.second;
 			msf.cdmsf_frame0 = status->absolute.frame;
 		} else {
-			if (start_track >= 0) {
+			if (start_track > 0 &&
+			    priv && priv->track_info &&
+			    start_track <= priv->number_tracks ) {
 				GnomeCDRomMSF tmpmsf;
 				
 				add_msf (&priv->track_info[start_track - 1].address, start, &tmpmsf);
@@ -491,11 +499,20 @@ linux_cdrom_play (GnomeCDRom *cdrom,
 		}
 
 		if (finish == NULL) {
-			msf.cdmsf_min1 = priv->track_info[priv->number_tracks].address.minute;
-			msf.cdmsf_sec1 = priv->track_info[priv->number_tracks].address.second;
-			msf.cdmsf_frame1 = priv->track_info[priv->number_tracks].address.frame;
+			if ( priv && priv->track_info && 
+			    priv->number_tracks >0 ) {
+				msf.cdmsf_min1 = priv->track_info[priv->number_tracks].address.minute;
+				msf.cdmsf_sec1 = priv->track_info[priv->number_tracks].address.second;
+				msf.cdmsf_frame1 = priv->track_info[priv->number_tracks].address.frame;
+			} else {
+				msf.cdmsf_min1 = 0;
+				msf.cdmsf_sec1 = 0;
+				msf.cdmsf_frame1 = 0;
+			}
 		} else {
-			if (finish_track >= 0) {
+			if (finish_track > 0 &&
+			    priv && priv->track_info &&
+			    finish_track <= priv->number_tracks ) {
 				GnomeCDRomMSF tmpmsf;
 
 				add_msf (&priv->track_info[finish_track - 1].address, finish, &tmpmsf);
@@ -640,7 +657,6 @@ linux_cdrom_stop (GnomeCDRom *cdrom,
 	}
 
 	linux_cdrom_close (lcd);
-	g_free (status);
 	return TRUE;
 }
 
@@ -773,7 +789,8 @@ linux_cdrom_back (GnomeCDRom *cdrom,
 	return TRUE;
 }
 
-/* There should probably be 2 get_status functions. A public one and the private one.
+/* There should probably be 2 get_status functions.
+   A public one and the private one.
    The private one would get called by the update handler every second, and the
    public one would just return a copy of the status */
 static gboolean
@@ -785,8 +802,12 @@ linux_cdrom_get_status (GnomeCDRom *cdrom,
 	LinuxCDRomPrivate *priv;
 	GnomeCDRomStatus *realstatus;
 	struct cdrom_subchnl subchnl;
+	struct cdrom_tocentry tocentry;
 	struct cdrom_volctrl vol;
 	int cd_status;
+	int i,j;
+	/* bug 117695: set this to FALSE if this ioctl is not supported */
+	static gboolean CDROMVOLREAD_supported = TRUE;
 	
 	g_return_val_if_fail (status != NULL, TRUE);
 	
@@ -840,7 +861,28 @@ linux_cdrom_get_status (GnomeCDRom *cdrom,
 			return TRUE;
 
 		default:
-			realstatus->cd = GNOME_CDROM_STATUS_OK;
+			if (ioctl (cdrom->fd, CDROMREADTOCHDR, priv->tochdr) < 0) {
+				g_print ("Error reading CD header");
+				linux_cdrom_close (lcd);
+
+				return;
+			}
+			realstatus->cd = GNOME_CDROM_STATUS_DATA_CD;
+
+			for (i = 0, j = priv->tochdr->cdth_trk0; i < (priv->tochdr->cdth_trk1 - priv->tochdr->cdth_trk0 + 1); i++, j++) {
+				tocentry.cdte_track = j;
+				tocentry.cdte_format = CDROM_MSF;
+
+				if (ioctl (cdrom->fd, CDROMREADTOCENTRY, &tocentry) < 0) {
+					g_print ("IOCtl failed");
+					continue;
+				}
+
+				if (tocentry.cdte_ctrl != CDROM_DATA_TRACK) {
+					realstatus->cd = GNOME_CDROM_STATUS_OK;
+					break; 
+				}
+			}
 			break;
 		}
 	} else {
@@ -873,18 +915,28 @@ linux_cdrom_get_status (GnomeCDRom *cdrom,
 	}
 
 	/* Get the volume */
-	if (ioctl (cdrom->fd, CDROMVOLREAD, &vol) < 0) {
-		g_warning ("(linux_cdrom_get_status): CDROMVOLREAD ioctl failed %s",
-			   g_strerror (errno));
-		realstatus->volume = -1; /* -1 means no volume command */
-	} else {
-		realstatus->volume = vol.channel0;
+	if (CDROMVOLREAD_supported) {
+		if (ioctl (cdrom->fd, CDROMVOLREAD, &vol) < 0) {
+			/* if ENOTSUP, flag so we don't repeatedly call */
+			if (errno == ENOTSUP) {
+				g_warning ("(linux_cdrom_get_status): CDROMVOLREAD ioctl not supported.");
+				CDROMVOLREAD_supported = FALSE;
+			} else {
+				g_warning ("(linux_cdrom_get_status): CDROMVOLREAD ioctl failed %s",
+					g_strerror (errno));
+				realstatus->volume = -1; /* -1 means no volume command */
+			}
+		} else {
+			realstatus->volume = vol.channel0;
+		}
 	}
 
 	linux_cdrom_close (lcd);
 
 	ASSIGN_MSF (realstatus->relative, blank_msf);
 	ASSIGN_MSF (realstatus->absolute, blank_msf);
+	ASSIGN_MSF (realstatus->length, blank_msf);
+	
 	realstatus->track = 1;
 	switch (subchnl.cdsc_audiostatus) {
 	case CDROM_AUDIO_PLAY:
@@ -892,7 +944,11 @@ linux_cdrom_get_status (GnomeCDRom *cdrom,
 		ASSIGN_MSF (realstatus->relative, subchnl.cdsc_reladdr.msf);
 		ASSIGN_MSF (realstatus->absolute, subchnl.cdsc_absaddr.msf);
 		realstatus->track = subchnl.cdsc_trk;
-
+		if(priv && realstatus->track>0 && 
+		   realstatus->track<=priv->number_tracks){
+			// track_info may not be initialized
+			ASSIGN_MSF (realstatus->length, priv->track_info[realstatus->track-1].length);
+		}
 		break;
 
 	case CDROM_AUDIO_PAUSED:
@@ -900,14 +956,23 @@ linux_cdrom_get_status (GnomeCDRom *cdrom,
 		ASSIGN_MSF (realstatus->relative, subchnl.cdsc_reladdr.msf);
 		ASSIGN_MSF (realstatus->absolute, subchnl.cdsc_absaddr.msf);
 		realstatus->track = subchnl.cdsc_trk;
-
+		if(priv && realstatus->track>0 && 
+		   realstatus->track<=priv->number_tracks){
+			// track_info may not be initialized
+			ASSIGN_MSF (realstatus->length, priv->track_info[realstatus->track-1].length);
+		}
 		break;
 
 	case CDROM_AUDIO_COMPLETED:
 		realstatus->audio = GNOME_CDROM_AUDIO_COMPLETE;
 		ASSIGN_MSF (realstatus->relative, subchnl.cdsc_reladdr.msf);
 		ASSIGN_MSF (realstatus->absolute, subchnl.cdsc_absaddr.msf);
-		realstatus->track = subchnl.cdsc_trk;		
+		realstatus->track = subchnl.cdsc_trk;
+		if(priv && realstatus->track>0 && 
+		   realstatus->track<=priv->number_tracks){
+			// track_info may not be initialized
+			ASSIGN_MSF (realstatus->length, priv->track_info[realstatus->track-1].length);
+		}
 		break;
 		
 	case CDROM_AUDIO_INVALID:
@@ -1011,7 +1076,7 @@ linux_cdrom_is_cdrom_device (GnomeCDRom *cdrom,
 	}
 	
 	close (fd);
-
+	
 	return TRUE;
 }
 
@@ -1065,7 +1130,7 @@ linux_cdrom_class_init (LinuxCDRomClass *klass)
 {
 	GObjectClass *object_class;
 	GnomeCDRomClass *cdrom_class;
-
+	
 	object_class = G_OBJECT_CLASS (klass);
 	cdrom_class = GNOME_CDROM_CLASS (klass);
 
